/****************************************************************************
 *   L9110.h by (c) 2019 Marcio Teixeira                                    *
 *                                                                          *
 *   This program is free software: you can redistribute it and/or modify   *
 *   it under the terms of the GNU General Public License as published by   *
 *   the Free Software Foundation, either version 3 of the License, or      *
 *   (at your option) any later version.                                    *
 *                                                                          *
 *   This program is distributed in the hope that it will be useful,        *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of         *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          *
 *   GNU General Public License for more details.                           *
 *                                                                          *
 *   To view a copy of the GNU General Public License, go to the following  *
 *   location: <http://www.gnu.org/licenses/>.                              *
 ****************************************************************************/
 
class L9110 {
  private:
    uint8_t a, b, pwm;

    void setOutput(uint8_t pin, bool state, uint8_t speed = 255) {
      if(pin == pwm)
        analogWrite(pin, state ? speed : (255 - speed));
      else
        digitalWrite(pin, state);
    }
  public:
    L9110(uint8_t ia, uint8_t ib) : a(ia), b(ib) {
      if(digitalPinToTimer(ia) != NOT_ON_TIMER)
        pwm = ia;
      if(digitalPinToTimer(ib) != NOT_ON_TIMER)
        pwm = ib;
      pinMode(a, OUTPUT);
      pinMode(b, OUTPUT);
    }


    void stop() {
      setOutput(a, LOW);
      setOutput(b, LOW);
    }

    void forward(float speed = 1.0) {
      setOutput(a, HIGH, speed * 255);
      setOutput(b, LOW,  speed * 255);      
    }

    void reverse(float speed = 1.0) {
      setOutput(a, LOW,  speed * 255);
      setOutput(b, HIGH, speed * 255);   
    }
};