#include <SPI.h>
#include <nRF24L01.h>
#include <RF24.h>
#include <printf.h>

#include "src/lib/Radio/MiLightRadioConfig.h"
#include "src/lib/Radio/NRF24MiLightRadio.h"
#include "src/RgbwPacketDecoder.h"
#include "src/L9110.h"

#define CE_PIN 9
#define CSN_PIN 10

// Driver Pins

#define IB1 5
#define IA1 A1

#define IB2 6
#define IA2 A2

enum {
  RGBW    = 0,
  CCT     = 1,
  RGB_CCT = 2,
  RGB     = 3
};

MiLightRadioConfig &config = MiLightRadioConfig::ALL_CONFIGS[RGBW];

RF24 radio(CE_PIN, CSN_PIN);
NRF24MiLightRadio mlr(radio, config);

RgbwPacketDecoder decoder;

void setup() {
  Serial.begin(115200);
  printf_begin();
  printf("Starting\n");
  if(mlr.begin())
    printf("Error opening radio\n");
  printf("Listening...\n");
}

L9110 left (IB2,IA2);
L9110 right(IA1,IB1);

void process_remote() {
  button_t button = decoder.getButton();
  Serial.print("Button: ");
  Serial.print(button);
  Serial.print("  Brightness: ");
  Serial.print(decoder.getBrightness());

  Serial.print("  Hue: ");
  Serial.print(decoder.getHue());

  switch(button) {
    case 3:  left.forward(); break;
    case 4:  left.reverse(); break;
    case 5:  left.forward(); right.forward(); break;
    case 6:  left.reverse(); right.reverse(); break;
    case 7:  left.forward(); right.forward(); break;
    case 8:  left.reverse(); right.reverse(); break;
    case 9:  right.forward(); break;
    case 10: right.reverse(); break;
    case 15: {
      const int angle = (decoder.getHue() + 130) % 360;
      const float turn = sin(angle * PI / 180);
      const float left_speed  = 1 - max(0,  turn);
      const float right_speed = 1 - max(0, -turn);
      if(angle < 90 || angle > 270) {
        left.forward(left_speed);
        right.forward(right_speed);
      } else {
        left.reverse(left_speed);
        right.reverse(right_speed);
      }
    }
  }
}

void loop() {
  static long lastMotion = millis();
  
  if (mlr.available()) {
    printf("\n");
    size_t packet_length = config.packetLength;
    uint8_t packet[packet_length];
    mlr.read(packet, packet_length);

    for (int i = 0; i < packet_length; i++) {
      printf("%02X ", packet[i]);
    }

    decoder.decode(packet, packet_length);
    process_remote();

    lastMotion = millis();
  }

  if(millis() - lastMotion > 400) {
    left.stop(); right.stop();
  }
}
